"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.spawnPromise = exports.plugin = void 0;
const framework_core_1 = require("@cloudbase/framework-core");
const fs_extra_1 = __importDefault(require("fs-extra"));
const axios_1 = __importDefault(require("axios"));
const compressing_1 = __importDefault(require("compressing"));
const path_1 = __importDefault(require("path"));
const os_1 = __importDefault(require("os"));
const child_process_1 = require("child_process");
const PLUGIN_DIR = path_1.default.join(os_1.default.homedir(), 'cloudbase-framework/registry/node_modules/@cloudbase/framework-plugin-low-code@inline');
class LowCodeIntallerPlugin extends framework_core_1.Plugin {
    constructor(name, api, inputs) {
        super(name, api, inputs);
        this.name = name;
        this.api = api;
        this.inputs = inputs;
        const { url = 'https://ci-source-1303824488.cos.ap-shanghai.myqcloud.com/presetResource/plugin.tar.gz' } = inputs, rest = __rest(inputs, ["url"]);
        this._lowcodeUrl = url;
        this._inputs = rest;
    }
    _subPluginConstructor() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._loadLowCodePlugin(PLUGIN_DIR);
            const { plugin: LowcodePlugin } = require(PLUGIN_DIR);
            this._lowcodePlugin = new LowcodePlugin('lowcode', this.api, this._inputs);
        });
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    remove() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    genCode() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._subPluginConstructor();
            yield this._lowcodePlugin.init();
            return this._lowcodePlugin.build();
        });
    }
    compile() {
        return __awaiter(this, void 0, void 0, function* () {
            return this._lowcodePlugin.compile();
        });
    }
    deploy() {
        return __awaiter(this, void 0, void 0, function* () {
            return this._lowcodePlugin.deploy();
        });
    }
    _loadLowCodePlugin(pluginDir) {
        return __awaiter(this, void 0, void 0, function* () {
            const source = this._lowcodeUrl;
            fs_extra_1.default.emptyDirSync(pluginDir);
            const isExist = fs_extra_1.default.existsSync(path_1.default.join(pluginDir, 'package.json'));
            if (isExist && process.env.CLOUDBASE_FX_ENV === 'dev') {
                this.api.logger.info('CLOUDBASE_FX_ENV=dev时，进入本地开发模式，插件使用本地link版本');
                return true;
            }
            try {
                yield downloadAndUncompress(source, pluginDir, /^cloudbase-/.test(process.env.CCI_CURRENT_TEAM || ''));
            }
            catch (e) {
                console.error(`Fail to download lowcode plugin package ${source}`, e);
                throw e;
            }
        });
    }
}
function downloadAndUncompress(source, target, useBash) {
    return __awaiter(this, void 0, void 0, function* () {
        if (useBash) {
            const processTag = `bash ${source}`;
            console.time(processTag);
            yield spawnPromise(`wget -qO- ${source} | tar xvz -C ${target}`, [], {
                cwd: process.cwd(),
                stdio: undefined,
            });
            console.timeEnd(processTag);
        }
        else {
            const downloadTag = `download ${source}`;
            console.time(downloadTag);
            let response = yield (0, axios_1.default)({
                url: source,
                responseType: 'stream',
            });
            console.timeEnd(downloadTag);
            const tag = `uncompress ${source}`;
            console.time(tag);
            yield compressing_1.default.tgz.uncompress(response.data, target);
            console.timeEnd(tag);
        }
    });
}
exports.plugin = LowCodeIntallerPlugin;
function spawnPromise(command, args, options) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            var _a, _b;
            const cm = (0, child_process_1.spawn)(command, args, Object.assign({
                shell: true,
                stdio: 'inherit',
            }, options));
            let stdout = '';
            (_a = cm.stdout) === null || _a === void 0 ? void 0 : _a.on('data', (data) => {
                stdout += data;
            });
            let stderr = '';
            (_b = cm.stderr) === null || _b === void 0 ? void 0 : _b.on('data', (data) => {
                stderr += data;
            });
            cm.on('error', reject);
            cm.on('close', (code) => (code === 0 ? resolve(stdout) : reject(stderr)));
        });
    });
}
exports.spawnPromise = spawnPromise;
